extends CharacterBody2D

const MOVE_SPEED = 50
const DECAY_MOVE_SPEED = 0.9 
const GRAVITY = 50
const JUMP_POWER = 1000
const JUMP_SCALE_TIME := 0.2
const JUMP_SCALE_VAL_JUMP := 0.3
const JUMP_SCALE_VAL_LANDING := 0.25

# ジャンプによるスケール状態
enum eJumpScale {
	NONE,
	JUMPING, #ジャンプ開始
	LANDING, # 着地開始
}

@onready var _spr = $Player

var _is_jumping = false # ジャンプ中かどうか
var _is_directon_left = false # 左向きかどうか.
var _tAnim:float = 0 # アニメーションタイマー.
var _jump_scale = eJumpScale.NONE
var _jump_scale_timer = 0

func _physics_process(delta: float) -> void:
	# 重力を加算
	velocity.y += GRAVITY
	
	# 移動量を減衰.
	velocity.x *= DECAY_MOVE_SPEED
	
	# 左右移動.
	if Input.is_action_pressed("ui_left"):
		velocity.x -= MOVE_SPEED # 左方向に移動
	elif Input.is_action_pressed("ui_right"):
		velocity.x += MOVE_SPEED # 右方向に移動.
	
	# ジャンプ.
	if _is_jumping == false:
		# ジャンプ中でなければジャンプできる.
		if Input.is_action_just_pressed("ui_accept"):
			# SPACEキーでジャンプする.
			velocity.y = -JUMP_POWER
			
			# ジャンプ開始.
			_jump_scale = eJumpScale.JUMPING
			_jump_scale_timer = JUMP_SCALE_TIME	
	# 移動処理.
	move_and_slide()
	
	if is_on_floor():
		if _is_jumping:
			# 着地した直後.
			_jump_scale = eJumpScale.LANDING
			_jump_scale_timer = JUMP_SCALE_TIME
		_is_jumping = false # 床に着地している
	else:
		_is_jumping = true # ジャンプ中.

func _process(delta: float) -> void:
	_tAnim += delta
	
	# アニメーションの切り替え.
	if velocity.x < 0:
		_is_directon_left = true # 左向き
	elif velocity.x > 0:
		_is_directon_left = false # 右向き

	_spr.flip_h = _is_directon_left
	
	var tbl = [0, 1];
	var t = _tAnim
	if abs(velocity.x) > 50:
		# 速度が50以上なら走り中.
		tbl = [2, 3]
		t *= 8 # アニメーション速度を上げる

	t = int(t) % 2
	_spr.frame = tbl[t]
	
	if _is_jumping:
		# ジャンプ中は2番目固定.
		_spr.frame = 2
	
	# ジャンプ・着地演出
	_update_jump_scale_anim(delta)

# ジャンプ・着地によるスケールアニメーションの更新
func _update_jump_scale_anim(delta:float) -> void:
	_jump_scale_timer -= delta
	if _jump_scale_timer <= 0:
		# 演出終了
		_jump_scale = eJumpScale.NONE
	match _jump_scale:
		eJumpScale.JUMPING:
			# 縦に伸ばす
			var d = JUMP_SCALE_VAL_JUMP * cube_in_out(_jump_scale_timer / JUMP_SCALE_TIME)
			_spr.scale.x = 1 - d
			_spr.scale.y = 1 + d * 3
		eJumpScale.LANDING:
			# 縦に潰す
			var d = JUMP_SCALE_VAL_LANDING * back_in_out(_jump_scale_timer / JUMP_SCALE_TIME)
			_spr.scale.x = 1 + d
			_spr.scale.y = 1 - d * 1.5
		_:
			# もとに戻す
			_spr.scale.x = 1
			_spr.scale.y = 1

# 三次関数
func cube_in(t:float) -> float:
	return t * t * t
func cube_out(t:float) -> float:
	return 1 + (t - 1) * (t - 1) * (t - 1)
func cube_in_out(t:float) -> float:
	if t <= 0.5:
		return t * t * t * 4
	else :
		return 1 + (t - 1) * (t - 1) * (t - 1) * 4

# back
func back_in(t:float) -> float:
	return t * t * (2.70158 * t - 1.70158)
func back_out(t:float) -> float:
	return 1 - (t - 1) * (t-1) * (-2.70158 * (t-1) - 1.70158)
func back_in_out(t:float) -> float:
	t *= 2
	if (t < 1):
		return t * t * (2.70158 * t - 1.70158) / 2
	else:
		t -= 1
		return (1 - (t - 1) * (t - 1) * (-2.70158 * (t - 1) - 1.70158)) / 2 + .5
